@@ -1,15 +1,19 @@
 from Acquisition import aq_inner
 from ftw.solr import _
+from ftw.solr import IS_PLONE_5
+from ftw.solr.browser.search import prepare_SearchableText
 from ftw.solr.interfaces import ILiveSearchSettings
 from itertools import groupby
 from plone import api
 from plone.registry.interfaces import IRegistry
 from Products.CMFCore.utils import getToolByName
 from Products.CMFPlone.browser.navtree import getNavigationRoot
+from Products.CMFPlone.utils import safe_hasattr
 from Products.CMFPlone.utils import safe_unicode
 from Products.Five.browser import BrowserView
 from Products.PythonScripts.standard import html_quote
 from Products.PythonScripts.standard import url_quote_plus
+from urllib import urlencode
 from zope.component import getMultiAdapter
 from zope.component import getUtility
 from zope.i18n import translate
@@ -20,13 +24,6 @@ def quotestring(s):
     return '"%s"' % s
 
 
-def quote_bad_chars(s):
-    bad_chars = ["(", ")"]
-    for char in bad_chars:
-        s = s.replace(char, quotestring(char))
-    return s
-
-
 class FtwSolrLiveSearchReplyView(BrowserView):
 
     def __call__(self):
@@ -38,8 +35,13 @@ class FtwSolrLiveSearchReplyView(BrowserView):
         plone_utils = getToolByName(context, 'plone_utils')
         self.pretty_title_or_id = plone_utils.pretty_title_or_id
         self.normalizeString = plone_utils.normalizeString
-        plone_view = getMultiAdapter((context, self.request), name='plone')
-        self.get_icon = plone_view.getIcon
+
+        if IS_PLONE_5:
+            layout_view = getMultiAdapter((context, self.request), name='plone_layout')
+            self.get_icon = layout_view.getIcon
+        else:
+            plone_view = getMultiAdapter((context, self.request), name='plone')
+            self.get_icon = plone_view.getIcon
 
         pprops = getToolByName(context, 'portal_properties')
         sprops = getattr(pprops, 'site_properties', None)
@@ -51,46 +53,50 @@ class FtwSolrLiveSearchReplyView(BrowserView):
         registry = getUtility(IRegistry)
         self.settings = registry.forInterface(ILiveSearchSettings)
 
-        # XXX really if it contains + * ? or -
-        # it will not be right since the catalog ignores all non-word
-        # characters equally like
-        # so we don't even attept to make that right.
-        # But we strip these and these so that the catalog does
-        # not interpret them as metachars
-        # See http://dev.plone.org/plone/ticket/9422 for an explanation of
-        # '\u3000'
-        multispace = u'\u3000'.encode('utf-8')
-        for char in ('?', '-', '+', '*', multispace):
-            q = q.replace(char, ' ')
-        r = quote_bad_chars(q) + '*'
+        q = prepare_SearchableText(q)
+        r = q + '*'
         self.searchterms = url_quote_plus(r)
 
         site_encoding = plone_utils.getSiteEncoding()
         if path is None:
             path = getNavigationRoot(context)
-        catalog = getToolByName(context, 'portal_catalog')
         friendly_types = plone_utils.getUserFriendlyTypes()
 
         self.facet_params = context.restrictedTraverse(
             '@@search-facets/facet_parameters')()
 
-        self.searchterm_query = '?searchterm=%s' % url_quote_plus(q)
+        self.searchterm_query = 'searchterm=%s' % url_quote_plus(q)
 
+        sort_limit = self.limit
         if self.settings.grouping:
-            results = catalog(SearchableText=r, portal_type=friendly_types,
-                              request_handler='livesearch', path=path,
-                              sort_limit=self.settings.group_search_limit)[:self.settings.group_search_limit]
+            sort_limit = self.settings.group_search_limit
 
-        else:
-            results = catalog(SearchableText=r, portal_type=friendly_types,
-                              request_handler='livesearch', path=path,
-                              sort_limit=self.limit)[:self.limit]
+        self.solr_response = self.get_solr_response(
+            SearchableText=r,
+            portal_type=friendly_types,
+            request_handler='livesearch',
+            path=path,
+            sort_limit=sort_limit,
+        )
+        results = self.solr_response[:sort_limit]
 
         self.request.response.setHeader(
             'Content-Type', 'application/json;charset=%s' % site_encoding)
 
         return self.get_results_as_json(results)
 
+    def get_solr_response(self, **query_params):
+        context = aq_inner(self.context)
+        catalog = getToolByName(context, 'portal_catalog')
+        query_params = self.get_query_params(**query_params)
+        return catalog(**query_params)
+
+    def get_query_params(self, **query_params):
+        """
+        Subclasses may inject more query params by overwriting this method.
+        """
+        return query_params
+
     def get_results_as_json(self, results):
         if self.settings.grouping:
             grouped_results = []
@@ -129,7 +135,9 @@ class FtwSolrLiveSearchReplyView(BrowserView):
             url = result.getURL()
             if result.portal_type in self.useViewAction:
                 url += '/view'
-            url = url + self.searchterm_query
+
+            separator = '&' if '?' in url else '?'
+            url = url + separator + self.searchterm_query
             title = self.get_title(result)
             first_of_group = 'first_of_group' in result and result.first_of_group
 
@@ -195,7 +203,7 @@ class FtwSolrLiveSearchReplyView(BrowserView):
             'title': translate(label_search_help, context=self.request),
             'first_of_group': False,
             'cssclass': 'no-result'
-            }
+        }
 
     def get_show_more_item(self):
         label_show_all = _('label_show_all', default='Show all items')
@@ -216,8 +224,21 @@ class FtwSolrLiveSearchReplyView(BrowserView):
     def get_nothing_found_item(self):
         label_nothing_found = _('label_nothing_found',
                                 default='No items found')
+        label_suggestion = ''
         params = self.facet_params
-        params += '&SearchableText=' + self.searchterms
+
+        # Use first suggestion in not found label.
+        suggestion = self.first_suggestion()
+        if suggestion:
+            word, suggestions_params = suggestion[0]
+            label_suggestion = _(
+                'label_nothing_found_suggestions',
+                default='Did you mean: ${word}',
+                mapping={'word': u'<span class="suggestion">{}</span>'.format(word)})
+            params += suggestions_params
+        else:
+            params += '&SearchableText=' + self.searchterms
+
         path = self.request.form.get('path', None)
         if path:
             params += '&path=' + url_quote_plus(path)
@@ -226,6 +247,35 @@ class FtwSolrLiveSearchReplyView(BrowserView):
         return {
             'url': url,
             'title': translate(label_nothing_found, context=self.request),
+            'description': translate(label_suggestion, context=self.request),
             'first_of_group': False,
-            'cssclass': 'no-result'
+            'cssclass': 'no-result',
         }
+
+    def first_suggestion(self):
+        """Get suggestions from spellcheck component.
+        This a copy of the search.py suggestions method.
+        """
+
+        if not safe_hasattr(self, 'solr_response'):
+            return []
+
+        suggested_terms = []
+        search_terms = [term.lower() for term in self.request.get(
+            'term', '').split()]
+        query_params = self.request.form.copy()
+
+        if safe_hasattr(self.solr_response, 'spellcheck'):
+            suggestions = self.solr_response.spellcheck.get('suggestions', [])
+            for term in search_terms:
+
+                if not isinstance(term, unicode):
+                    term = term.decode('utf-8')
+
+                if term in suggestions:
+                    suggestion = suggestions[term]['suggestion']
+                    query_params['SearchableText'] = suggestion[0]['word']
+                    query_string = '&' + urlencode(query_params.items())
+                    suggested_terms.append((query_params['SearchableText'],
+                                            query_string))
+        return suggested_terms
